@@ -2,6 +2,8 @@
 Dynamic dependency injection system for MCP Mesh.
 
 Handles both initial injection and runtime updates when topology changes.
+Focused purely on dependency injection - telemetry/tracing is handled at 
+the HTTP middleware layer for unified approach across MCP agents and FastAPI apps.
 """
 
 import asyncio
@@ -126,6 +128,9 @@ class DependencyInjector:
                 for func_id in self._dependency_mapping[name]:
                     if func_id in self._function_registry:
                         func = self._function_registry[func_id]
+                        logger.debug(
+                            f"🔄 UPDATING dependency '{name}' for {func_id} -> {func} at {hex(id(func))}"
+                        )
                         if hasattr(func, "_mesh_update_dependency"):
                             func._mesh_update_dependency(name, instance)
 
@@ -279,82 +284,223 @@ class DependencyInjector:
         # Capture logger in local scope to avoid NameError
         wrapper_logger = logger
 
-        @functools.wraps(func)
-        def dependency_wrapper(*args, **kwargs):
-            wrapper_logger.debug(
-                f"🔧 DEPENDENCY_WRAPPER: Function {func.__name__} called"
+        # If no mesh positions to inject, create minimal wrapper for tracking
+        if not mesh_positions:
+            logger.debug(
+                f"🔧 No injection positions for {func.__name__}, creating minimal wrapper for tracking"
             )
-            wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: args={args}, kwargs={kwargs}")
-            wrapper_logger.debug(
-                f"🔧 DEPENDENCY_WRAPPER: mesh_positions={mesh_positions}"
-            )
-            wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: dependencies={dependencies}")
 
-            # If no mesh positions to inject into, call original function
-            if not mesh_positions:
-                wrapper_logger.debug(
-                    "🔧 DEPENDENCY_WRAPPER: No mesh positions, calling original"
-                )
-                return func._mesh_original_func(*args, **kwargs)
-
-            # Get function signature
-            sig = inspect.signature(func)
-            params = list(sig.parameters.keys())
-            final_kwargs = kwargs.copy()
-
-            wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: params={params}")
-            wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: original kwargs={kwargs}")
-
-            # Inject dependencies as kwargs
-            injected_count = 0
-            for dep_index, param_position in enumerate(mesh_positions):
-                if dep_index < len(dependencies):
-                    dep_name = dependencies[dep_index]
-                    param_name = params[param_position]
-
-                    wrapper_logger.debug(
-                        f"🔧 DEPENDENCY_WRAPPER: Processing dep {dep_index}: {dep_name} -> {param_name}"
+            # Check if we need async wrapper for minimal case
+            if inspect.iscoroutinefunction(func):
+                @functools.wraps(func)
+                async def minimal_wrapper(*args, **kwargs):
+                    # Use ExecutionTracer for functions without dependencies (v0.4.0 style)
+                    from ..tracing.execution_tracer import ExecutionTracer
+                    wrapper_logger.debug(f"🔧 DI: Executing async function {func.__name__} (no dependencies)")
+                    
+                    # For async functions without dependencies, use the async tracer
+                    return await ExecutionTracer.trace_function_execution_async(
+                        func, args, kwargs, [], [], 0, wrapper_logger
                     )
+            else:
+                @functools.wraps(func)
+                def minimal_wrapper(*args, **kwargs):
+                    # Use ExecutionTracer for functions without dependencies (v0.4.0 style)
+                    from ..tracing.execution_tracer import ExecutionTracer
+                    wrapper_logger.debug(f"🔧 DI: Executing sync function {func.__name__} (no dependencies)")
+                    
+                    # Use original function tracer for functions without dependencies
+                    return ExecutionTracer.trace_original_function(func, args, kwargs, wrapper_logger)
+
+            # Add minimal metadata for compatibility
+            minimal_wrapper._mesh_injected_deps = {}
+            minimal_wrapper._mesh_dependencies = dependencies
+            minimal_wrapper._mesh_positions = mesh_positions
+            minimal_wrapper._mesh_parameter_types = get_agent_parameter_types(func)
+            minimal_wrapper._mesh_original_func = func
+
+            def update_dependency(name: str, instance: Any | None) -> None:
+                """No-op update for functions without injection positions."""
+                pass
+
+            minimal_wrapper._mesh_update_dependency = update_dependency
+
+            # Register this wrapper for dependency updates (even though it won't use them)
+            logger.debug(
+                f"🔧 REGISTERING minimal wrapper: {func_id} -> {minimal_wrapper} at {hex(id(minimal_wrapper))}"
+            )
+            self._function_registry[func_id] = minimal_wrapper
+
+            return minimal_wrapper
+
+        # Determine if we need async wrapper
+        need_async_wrapper = inspect.iscoroutinefunction(func)
+
+        if need_async_wrapper:
+
+            @functools.wraps(func)
+            async def dependency_wrapper(*args, **kwargs):
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: Function {func.__name__} called"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: args={args}, kwargs={kwargs}"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: mesh_positions={mesh_positions}"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: dependencies={dependencies}"
+                )
+
+                # We know mesh_positions is not empty since we checked above
+
+                # Get function signature
+                sig = inspect.signature(func)
+                params = list(sig.parameters.keys())
+                final_kwargs = kwargs.copy()
+
+                wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: params={params}")
+                wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: original kwargs={kwargs}")
+
+                # Inject dependencies as kwargs
+                injected_count = 0
+                for dep_index, param_position in enumerate(mesh_positions):
+                    if dep_index < len(dependencies):
+                        dep_name = dependencies[dep_index]
+                        param_name = params[param_position]
 
-                    # Only inject if the parameter wasn't explicitly provided
-                    if (
-                        param_name not in final_kwargs
-                        or final_kwargs.get(param_name) is None
-                    ):
-                        # Get the dependency from wrapper's storage
-                        dependency = dependency_wrapper._mesh_injected_deps.get(
-                            dep_name
-                        )
                         wrapper_logger.debug(
-                            f"🔧 DEPENDENCY_WRAPPER: From wrapper storage: {dependency}"
+                            f"🔧 DEPENDENCY_WRAPPER: Processing dep {dep_index}: {dep_name} -> {param_name}"
                         )
 
-                        if dependency is None:
-                            dependency = self.get_dependency(dep_name)
+                        # Only inject if the parameter wasn't explicitly provided
+                        if (
+                            param_name not in final_kwargs
+                            or final_kwargs.get(param_name) is None
+                        ):
+                            # Get the dependency from wrapper's storage
+                            dependency = dependency_wrapper._mesh_injected_deps.get(
+                                dep_name
+                            )
                             wrapper_logger.debug(
-                                f"🔧 DEPENDENCY_WRAPPER: From global storage: {dependency}"
+                                f"🔧 DEPENDENCY_WRAPPER: From wrapper storage: {dependency}"
                             )
 
-                        final_kwargs[param_name] = dependency
-                        injected_count += 1
-                        wrapper_logger.debug(
-                            f"🔧 DEPENDENCY_WRAPPER: Injected {dep_name} as {param_name}"
-                        )
-                    else:
-                        wrapper_logger.debug(
-                            f"🔧 DEPENDENCY_WRAPPER: Skipping {param_name} - already provided"
-                        )
+                            if dependency is None:
+                                dependency = self.get_dependency(dep_name)
+                                wrapper_logger.debug(
+                                    f"🔧 DEPENDENCY_WRAPPER: From global storage: {dependency}"
+                                )
 
-            wrapper_logger.debug(
-                f"🔧 DEPENDENCY_WRAPPER: Injected {injected_count} dependencies"
-            )
-            wrapper_logger.debug(f"🔧 DEPENDENCY_WRAPPER: final_kwargs={final_kwargs}")
+                            final_kwargs[param_name] = dependency
+                            injected_count += 1
+                            wrapper_logger.debug(
+                                f"🔧 DEPENDENCY_WRAPPER: Injected {dep_name} as {param_name}"
+                            )
+                        else:
+                            wrapper_logger.debug(
+                                f"🔧 DEPENDENCY_WRAPPER: Skipping {param_name} - already provided"
+                            )
 
-            result = func._mesh_original_func(*args, **final_kwargs)
-            wrapper_logger.debug(
-                f"🔧 DEPENDENCY_WRAPPER: Original returned: {type(result)}"
-            )
-            return result
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: Injected {injected_count} dependencies"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: final_kwargs={final_kwargs}"
+                )
+
+                # ===== EXECUTE WITH DEPENDENCY INJECTION AND TRACING =====
+                # Use ExecutionTracer for comprehensive execution logging (v0.4.0 style)
+                from ..tracing.execution_tracer import ExecutionTracer
+                
+                original_func = func._mesh_original_func
+
+                wrapper_logger.debug(
+                    f"🔧 DI: Executing async function {original_func.__name__} with {injected_count} injected dependencies"
+                )
+
+                # Use ExecutionTracer's async method for clean tracing
+                result = await ExecutionTracer.trace_function_execution_async(
+                    original_func,
+                    args,
+                    final_kwargs,
+                    dependencies,
+                    mesh_positions,
+                    injected_count,
+                    wrapper_logger,
+                )
+
+                wrapper_logger.debug(
+                    f"🔧 DI: Function {original_func.__name__} returned: {type(result)}"
+                )
+                return result
+
+        else:
+            # Create sync wrapper for sync functions without dependencies
+            @functools.wraps(func)
+            def dependency_wrapper(*args, **kwargs):
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: Function {func.__name__} called"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: args={args}, kwargs={kwargs}"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: mesh_positions={mesh_positions}"
+                )
+                wrapper_logger.debug(
+                    f"🔧 DEPENDENCY_WRAPPER: dependencies={dependencies}"
+                )
+
+                # We know mesh_positions is not empty since we checked above
+
+                # Handle dependency injection for sync functions
+                sig = inspect.signature(func)
+                params = list(sig.parameters.keys())
+                final_kwargs = kwargs.copy()
+
+                # Inject dependencies as kwargs
+                injected_count = 0
+                for dep_index, param_position in enumerate(mesh_positions):
+                    if dep_index < len(dependencies):
+                        dep_name = dependencies[dep_index]
+                        param_name = params[param_position]
+
+                        # Only inject if the parameter wasn't explicitly provided
+                        if (
+                            param_name not in final_kwargs
+                            or final_kwargs.get(param_name) is None
+                        ):
+                            # Get the dependency from wrapper's storage
+                            dependency = dependency_wrapper._mesh_injected_deps.get(
+                                dep_name
+                            )
+
+                            if dependency is None:
+                                dependency = self.get_dependency(dep_name)
+
+                            final_kwargs[param_name] = dependency
+                            injected_count += 1
+
+                # ===== EXECUTE WITH DEPENDENCY INJECTION AND TRACING =====
+                # Use ExecutionTracer for comprehensive execution logging (v0.4.0 style)
+                from ..tracing.execution_tracer import ExecutionTracer
+                
+                wrapper_logger.debug(
+                    f"🔧 DI: Executing sync function {func._mesh_original_func.__name__} with {injected_count} injected dependencies"
+                )
+
+                # Use ExecutionTracer for clean execution tracing
+                return ExecutionTracer.trace_function_execution(
+                    func._mesh_original_func,
+                    args,
+                    final_kwargs,
+                    dependencies,
+                    mesh_positions,
+                    injected_count,
+                    wrapper_logger,
+                )
 
         # Store dependency state on wrapper
         dependency_wrapper._mesh_injected_deps = {}
@@ -382,6 +528,9 @@ class DependencyInjector:
         dependency_wrapper._mesh_original_func = func
 
         # Register this wrapper for dependency updates
+        logger.debug(
+            f"🔧 REGISTERING in function_registry: {func_id} -> {dependency_wrapper} at {hex(id(dependency_wrapper))}"
+        )
         self._function_registry[func_id] = dependency_wrapper
 
         # Return the wrapper (which FastMCP will register)
