@@ -7,6 +7,7 @@ import uuid
 from collections.abc import AsyncIterator
 from typing import Any, Optional
 
+from ..shared.sse_parser import SSEStreamProcessor
 from .async_mcp_client import AsyncMCPClient
 from .mcp_client_proxy import MCPClientProxy
 
@@ -44,6 +45,15 @@ class FullMCPProxy(MCPClientProxy):
                 f"🔧 FullMCPProxy initialized with kwargs: {self.kwargs_config}"
             )
 
+    def _inject_trace_headers(self, headers: dict) -> dict:
+        """Inject trace context headers for distributed tracing."""
+        from ..tracing.trace_context_helper import TraceContextHelper
+
+        TraceContextHelper.inject_trace_headers_to_request(
+            headers, self.endpoint, self.logger
+        )
+        return headers
+
     # Phase 6: Streaming Support - THE BREAKTHROUGH METHOD!
     async def call_tool_streaming(
         self, name: str, arguments: dict = None
@@ -75,30 +85,37 @@ class FullMCPProxy(MCPClientProxy):
             try:
                 import httpx
 
-                url = f"{self.endpoint}/mcp/"
+                url = f"{self.endpoint}/mcp"
+
+                # Build headers with trace context
+                headers = {
+                    "Content-Type": "application/json",
+                    "Accept": "text/event-stream",  # THIS IS THE KEY!
+                }
+                headers = self._inject_trace_headers(headers)
 
                 async with httpx.AsyncClient(timeout=30.0) as client:
                     async with client.stream(
                         "POST",
                         url,
                         json=payload,
-                        headers={
-                            "Content-Type": "application/json",
-                            "Accept": "text/event-stream",  # THIS IS THE KEY!
-                        },
+                        headers=headers,
                     ) as response:
                         if response.status_code >= 400:
                             raise RuntimeError(f"HTTP error {response.status_code}")
 
-                        async for line in response.aiter_lines():
-                            if line.startswith("data: "):
-                                try:
-                                    data_str = line[6:]  # Remove "data: " prefix
-                                    if data_str.strip():
-                                        chunk = json.loads(data_str)
-                                        yield chunk
-                                except json.JSONDecodeError:
-                                    continue
+                        # Use shared SSE stream processor
+                        sse_processor = SSEStreamProcessor(f"FullMCPProxy.{name}")
+
+                        async for chunk_bytes in response.aiter_bytes(8192):
+                            chunks = sse_processor.process_chunk(chunk_bytes)
+                            for chunk in chunks:
+                                yield chunk
+
+                        # Process any remaining data
+                        final_chunks = sse_processor.finalize()
+                        for chunk in final_chunks:
+                            yield chunk
 
             except ImportError:
                 # Fallback: if httpx not available, use sync call
@@ -217,7 +234,7 @@ class FullMCPProxy(MCPClientProxy):
             }
 
             # URL for MCP protocol endpoint
-            url = f"{self.endpoint.rstrip('/')}/mcp/"
+            url = f"{self.endpoint.rstrip('/')}/mcp"
 
             # Add session ID to headers for session routing
             headers = {
@@ -225,6 +242,7 @@ class FullMCPProxy(MCPClientProxy):
                 "Accept": "application/json, text/event-stream",  # Required by FastMCP
                 "X-Session-ID": session_id,  # Key header for session routing
             }
+            headers = self._inject_trace_headers(headers)
 
             async with httpx.AsyncClient() as client:
                 response = await client.post(url, json=payload, headers=headers)
@@ -479,7 +497,10 @@ class EnhancedFullMCPProxy(FullMCPProxy):
             else:
                 self.logger.warning("⚠️ Authentication required but no token available")
 
-        url = f"{self.endpoint}/mcp/"
+        # Inject trace context headers
+        headers = self._inject_trace_headers(headers)
+
+        url = f"{self.endpoint}/mcp"
 
         try:
             import httpx
@@ -556,7 +577,10 @@ class EnhancedFullMCPProxy(FullMCPProxy):
         if session_id:
             headers["X-Session-ID"] = session_id
 
-        url = f"{self.endpoint}/mcp/"
+        # Inject trace context headers
+        headers = self._inject_trace_headers(headers)
+
+        url = f"{self.endpoint}/mcp"
 
         try:
             import httpx
@@ -568,19 +592,20 @@ class EnhancedFullMCPProxy(FullMCPProxy):
                 ) as response:
                     response.raise_for_status()
 
-                    buffer = ""
-                    async for chunk in response.aiter_bytes(self.buffer_size):
-                        buffer += chunk.decode("utf-8")
+                    # Use shared SSE stream processor
+                    sse_processor = SSEStreamProcessor(f"EnhancedFullMCPProxy.{name}")
 
-                        while "\n" in buffer:
-                            line, buffer = buffer.split("\n", 1)
+                    async for chunk_bytes in response.aiter_bytes(
+                        max(self.buffer_size, 8192)
+                    ):
+                        chunks = sse_processor.process_chunk(chunk_bytes)
+                        for chunk in chunks:
+                            yield chunk
 
-                            if line.startswith("data: "):
-                                try:
-                                    data = json.loads(line[6:])
-                                    yield data
-                                except json.JSONDecodeError:
-                                    continue
+                    # Process any remaining data
+                    final_chunks = sse_processor.finalize()
+                    for chunk in final_chunks:
+                        yield chunk
 
         except httpx.TimeoutException:
             raise Exception(f"Streaming timeout after {self.stream_timeout}s")
