@@ -10,6 +10,7 @@ import uuid
 from typing import Any, Optional
 
 from ..shared.content_extractor import ContentExtractor
+from ..shared.sse_parser import SSEParser
 from .async_mcp_client import AsyncMCPClient
 
 logger = logging.getLogger(__name__)
@@ -92,40 +93,31 @@ class MCPClientProxy:
                 "params": {"name": self.function_name, "arguments": kwargs},
             }
 
-            url = f"{self.endpoint}/mcp/"  # Use trailing slash to avoid 307 redirect
+            url = f"{self.endpoint}/mcp"  # Remove trailing slash to avoid 307 redirect
             data = json.dumps(payload).encode("utf-8")
 
-            req = urllib.request.Request(
-                url,
-                data=data,
-                headers={
-                    "Content-Type": "application/json",
-                    "Accept": "application/json, text/event-stream",  # FastMCP requires both
-                },
+            # Build headers with trace context injection
+            headers = {
+                "Content-Type": "application/json",
+                "Accept": "application/json, text/event-stream",  # FastMCP requires both
+            }
+
+            # Inject trace headers for distributed tracing
+            from ..tracing.trace_context_helper import TraceContextHelper
+
+            TraceContextHelper.inject_trace_headers_to_request(
+                headers, url, self.logger
             )
 
+            req = urllib.request.Request(url, data=data, headers=headers)
+
             with urllib.request.urlopen(req, timeout=30.0) as response:
                 response_data = response.read().decode("utf-8")
 
-                # Handle Server-Sent Events format from FastMCP
-                if response_data.startswith("event:"):
-                    # Parse SSE format: extract JSON from "data:" lines
-                    json_data = None
-                    for line in response_data.split("\n"):
-                        if line.startswith("data:"):
-                            json_str = line[5:].strip()  # Remove 'data:' prefix
-                            try:
-                                json_data = json.loads(json_str)
-                                break
-                            except json.JSONDecodeError:
-                                continue
-
-                    if json_data is None:
-                        raise RuntimeError("Could not parse SSE response from FastMCP")
-                    data = json_data
-                else:
-                    # Plain JSON response
-                    data = json.loads(response_data)
+                # Use shared SSE parser
+                data = SSEParser.parse_sse_response(
+                    response_data, f"MCPClientProxy.{self.function_name}"
+                )
 
             # Check for JSON-RPC error
             if "error" in data:
@@ -297,6 +289,13 @@ class EnhancedMCPClientProxy(MCPClientProxy):
             # Add custom headers
             headers.update(self.custom_headers)
 
+            # Inject trace headers for distributed tracing
+            from ..tracing.trace_context_helper import TraceContextHelper
+
+            TraceContextHelper.inject_trace_headers_to_request(
+                headers, url, self.logger
+            )
+
             # Add authentication headers if required
             if self.auth_required:
                 auth_token = os.getenv("MCP_MESH_AUTH_TOKEN")
@@ -411,6 +410,13 @@ class EnhancedMCPClientProxy(MCPClientProxy):
             # Add custom headers
             headers.update(self.custom_headers)
 
+            # Inject trace headers for distributed tracing
+            from ..tracing.trace_context_helper import TraceContextHelper
+
+            TraceContextHelper.inject_trace_headers_to_request(
+                headers, url, self.logger
+            )
+
             # Add authentication headers if required
             if self.auth_required:
                 auth_token = os.getenv("MCP_MESH_AUTH_TOKEN")
